

.MEMORYMAP
  SLOTSIZE $4000
  DEFAULTSLOT 1
  SLOT 0 $0000
  SLOT 1 $4000
.ENDME

.ROMBANKSIZE $4000
.ROMBANKS 2

.EMPTYFILL $00
.CARTRIDGETYPE 1
.COMPUTECHECKSUM
.COMPUTECOMPLEMENTCHECK

.include "hw.inc"
.include "rst.inc"
.include "logo.inc"

.bank 0 slot 0

; ==========
; interrupts
; ==========

; vblank
.org $0040
jp _vblank
;reti

; LCDC status
.org $0048
reti

; timer overflow
.org $0050
ei ; nested interrupts (_vblank will be executed)
jp _timer
; reti

; serial transfer completion
.org $0058
 reti

; gamepad
.org $0060
reti

; ==========

; cartridge header: $0100 to $014f
.org $101
jp _start

.org $134
  .db "DEMO            " ; gamepak name (16 characters)
  .db $00,$00,$00        ; (?)
  .db $00                ; ROM only
  .db $00                ; ROM size (32kB)
  .db $00                ; RAM size (none)
  .dw $c0de              ; manufacturer code
  .db $01                ; version
  .db $00                ; header checksum (overwriten)
  .db $00,$00            ; global checksum (overwriten)

.org $150
_start:

 ;di
 ; configure interrupts
 ld a,%00000101
 ld (IE),a

 ; stack in HRAM
 ld sp, $fff0

 ; init work area in RAM to $00
 ; [debug]
 ; ...

 call _lcd_di

 ; disable sound
 ld a,$00
 ld (NR52),a

 ; LCD control configuration
 ld hl,LCDC
 ld a,(hl)
 and %10000000
 or %01010001
 ld (LCDC),a

 ; reset LCD position and scrolling
 ld a,$00
 ld (SCY),a
 ld (SCX),a
 ld (WY),a
 ld (WX),a

 ; joypad
 ld (P1),a

 ; palettes
 ; background
 ld a,%11100100
 ld (BGP),a
 ; sprites
 ld a,%11100100
 ld (OBP1),a
 ld a,%11100100
 ld (OBP0),a

 ; clear bg_map (VRAM / 9800h-9BFFh)
 ld hl,$9800   ; src
 ld d,$00      ; val
 ld bc,$400    ; len
 call _memset

 ; load 12 tiles in VRAM (8000h-8FFFh)
 ; 16 (= 1 tile) x 12 = 192 bytes
 ld hl,_bg_tile_data   ; src
 ld de,$8000           ; dst
 ld bc,192             ; len
 call _memcpy

 ; => move to WRAM BANK2...

 ; VRAM sprite attributes (OAM) (FE00h-FE9Fh)
 ; 40 entries x 4bytes
 ld a,$10
 ld ($FE00),a  ; y position
 ld a,$08
 ld ($FE01),a  ; x position
 ld a,$00
 ld ($FE02),a  ; tile #
 ld a,%00000000
 ld ($FE03),a  ; attributes

 ; enable OBJ (sprite) display
 ld hl,LCDC
 ld a,(hl)
 and %11111110 ; disable bg map
 or %00000010
 ld (LCDC),a

 ; VRAM only accessible during v-blank
 ; OAM mirrored in WRAM BANK2 (D000h-DFFFh) / sync'ed through DMA

 ; copy DMA_memcpy routine to HRAM
 ld hl,_hram_dma_memcpy      ; src
 ld de,$ff80                 ; dst
 ld bc,HRAM_DMA_MEMCPY_LEN   ; len
 call _memcpy

 ; clear OAM mirror area in WRAM BANK2
 ld hl,$d000   ; src
 ld d,$00      ; val
 ld bc,160     ; 160 = 40 entries x 4bytes
 call _memset

 ; set attributes for sprite #0
 ld a,$10
 ld ($d000),a  ; y position
 ld a,$08
 ld ($d001),a  ; x position
 ld a,$00
 ld ($d002),a  ; tile #
 ld a,$00
 ld ($d003),a  ; attributes

 ; configure TIMA (timer)

 ; 1s period = 4096Hz / ( 16 (WRAM variable) * 256 (TMA [timer modulo] = 0x00) )
 ld a,$00
 ld ($c000),a     ; clear WRAM (BANK1) timer variable
 ld a,$00
 ld ($FF06),a     ; TMA = 0x00 (TIMA will overflow after 256 ticks / 62.5ms)
 ld a,%00000100   ; timer start (bit 2) & set frequency (bit 1+0)
 ld ($FF07),a     ; TAC = 4096 Hz
 ei

 ; ...

 call _lcd_en

_l0:
 halt
 jp _l0

; ==============================

_timer:
 push af ; save context
 push hl ; save context

   ld hl,$c000 ; WRAM timer variable
   inc (hl)
   ld a,(hl)
   cp 16
   jp nz,_timer_exit ; = 0 ? (256 ticks = overflow) ?
   ld a,$00
   ld (hl),a

   ; ld hl,$FE02   ; tile #
   ld hl,$d002   ; tile # / WRAM BANK2
   ld a,9
   xor (hl)
   jp z,_timer_dpy_reset
   inc (hl)
   jp _timer_exit

_timer_dpy_reset:
   ld a,$00
   ; ld ($FE02),a  ; tile #
   ld ($d002),a  ; tile # / WRAM BANK2

_timer_exit:
 pop hl  ; restore context
 pop af  ; restore context   
 ret

; ==============================

_vblank:
 di
 push af ; save context
 push hl ; save context

   ld a,$d0    ; OAM mirror in WRAM BANK2 (D000h-D09Fh)
   call $ff80  ; DMA OAM copy (~200us)

 pop hl   ; restore context
 pop af   ; restore context
 reti

; ==============================

_lcd_en:
  ld hl,LCDC
  ld a,(hl)
  or %10000000 ; (run / bit7=1)
  ld (hl),a
  ; ei
  ret

_lcd_di:
  di
  ; vblank period starts after line 144 is drawn (LY >= 144)
  ld a,(LY)
  cp 145
  jr nz,_lcd_di
  ld hl,LCDC
  ld a,(hl)
  and %01111111 ; (stopped / bit7=0)
  ld (hl),a
  ; ei
  ret

; ==============================

 ; memcpy / copy [len] bytes from [src] to [dst]
 ; src = hl
 ; dst = de
 ; len = bc
_memcpy:
 ldi a,(hl)
 ld (de),a
 inc de
 dec bc
 ld a,c
 cp $00
 jp nz,_memcpy
 ld a,b
 cp $00
 jp nz,_memcpy
 ret

 ; memset / copy [val] into each of the first [len] bytes of the object pointed to by [src]
 ; src = hl
 ; val = d
 ; len = bc
_memset:
 ld (hl),d
 inc hl
 dec bc
 ld a,c
 cp $00
 jp nz,_memset
 ld a,b
 cp $00
 jp nz,_memset
 ret

.include "dma.inc"
.include "ascii_tile_data.inc"

